/**
*   Please feel free to use any fragment of the code in this file that you need
*   in your own work. As far as I am concerned, it's in the public domain. No
*   permission is necessary or required. Credit is always appreciated if you
*   use a large chunk or base a significant product on one of my examples,
*   but that's not required either.
*
*   This code is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
*      --- Joseph A. Huwaldt
**/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import net.java.games.jogl.*;

import jahuwaldt.gl.SoftFPSAnimator;


/**
*  A simple application to test the GeoMipMap landscape rendering algorithm.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author  Joseph A. Huwaldt   Date:  April 19, 2001
*  @version November 23, 2004
**/
public class GeoMMTest {
  
	private static LandscapeRenderer landScape = null;
	private static JTextField lodField = null;
	private static JTextField trisRendered = null;
	
	public static void main(String[] args) {
		try {
		
			JFrame frame = new JFrame("GeoMipMap Landscape Renderer");
			Container cp = frame.getContentPane();
			cp.setLayout(new BorderLayout());
			
			GLCanvas canvas = GLDrawableFactory.getFactory().createGLCanvas(new GLCapabilities());
			cp.add("Center", canvas);

			// Use debug pipeline
			//    canvas.setGL(new DebugGL(canvas.getGL()));
			System.err.println("CANVAS GL IS: " + canvas.getGL().getClass().getName());
			System.err.println("CANVAS GLU IS: " + canvas.getGLU().getClass().getName());

			//  Add the landscape renderer to the window.
			landScape = new LandscapeRenderer();
			canvas.addGLEventListener(landScape);
			frame.setSize(640, 600);
			final SoftFPSAnimator animator = new SoftFPSAnimator(canvas, 30);
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					animator.stop();
					System.exit(0);
				}
				public void windowActivated(WindowEvent e) {
					animator.start();
				}
				 
				public void windowDeactivated(WindowEvent e) {
					animator.stop();
				}

			});
			
			canvas.addKeyListener(new KeyAdapter() {
				public void keyPressed(KeyEvent e) {
					int key = e.getKeyCode();
					if ( key == 't' || key == 'T')
						System.out.println("\nfps = " + animator.getFPS());
				}
			});
			
			//  Add the controls to the window.
			cp.add("South", createControls());
			
			frame.show();
			animator.start();
			
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
	}

	private static JPanel createControls() {

		//	Create a panel to hold buttons on the bottom.
		JPanel buttonPanel = new JPanel();
		
		//	Create a checkbox group for drawing mode.
		JPanel drawModePanel = new JPanel(new GridLayout(0,1));
		drawModePanel.setBorder(BorderFactory.createTitledBorder(
								BorderFactory.createLineBorder(Color.black),
								"Drawing Mode"));
		ButtonGroup bg = new ButtonGroup();
		JRadioButton rb = new JRadioButton("Textured");
		rb.setSelected(true);
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setDrawMode(GeoMMLandscape.kUseTexture);
        		}
			});
		bg.add(rb);
		drawModePanel.add(rb);
		rb = new JRadioButton("Gaurad Shaded");
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setDrawMode(GeoMMLandscape.kUseGauradShading);
        		}
			});
		bg.add(rb);
		drawModePanel.add(rb);
		
		rb = new JRadioButton("Flat Shaded");
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setDrawMode(GeoMMLandscape.kUseFlatShading);
        		}
			});
		bg.add(rb);
		drawModePanel.add(rb);

		rb = new JRadioButton("Wireframe");
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setDrawMode(GeoMMLandscape.kUseWireFrame);
        		}
			});
		bg.add(rb);
		drawModePanel.add(rb);
		
		buttonPanel.add(drawModePanel);
		
		//	Create a checkbox group for camera mode.
		JPanel cameraModePanel = new JPanel(new GridLayout(0,1));
		cameraModePanel.setBorder(BorderFactory.createTitledBorder(
								BorderFactory.createLineBorder(Color.black),
								"Camera Mode"));
		bg = new ButtonGroup();
		rb = new JRadioButton("Follow Path");
		rb.setSelected(true);
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setCameraMode(LandscapeRenderer.FOLLOW_MODE);
        		}
			});
		bg.add(rb);
		cameraModePanel.add(rb);
		rb = new JRadioButton("Drive");
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setCameraMode(LandscapeRenderer.DRIVE_MODE);
        		}
			});
		bg.add(rb);
		cameraModePanel.add(rb);
		rb = new JRadioButton("Fly");
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setCameraMode(LandscapeRenderer.FLY_MODE);
        		}
			});
		bg.add(rb);
		cameraModePanel.add(rb);
		rb = new JRadioButton("Observe Algorithm");
		rb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.setCameraMode(LandscapeRenderer.OBSERVE_MODE);
        		}
			});
		bg.add(rb);
		cameraModePanel.add(rb);

		buttonPanel.add(cameraModePanel);
		
		//	Create a checkbox group for other options.
		JPanel optionsPanel = new JPanel(new GridLayout(0,1));
		optionsPanel.setBorder(BorderFactory.createTitledBorder(
								BorderFactory.createLineBorder(Color.black),
								"Options"));
		JCheckBox cb = new JCheckBox("Draw Field of View", true);
		cb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.drawFrustum(!landScape.drawFrustum());
        		}
			});
		optionsPanel.add(cb);
		cb = new JCheckBox("Draw Fog Effects", true);
		cb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		landScape.drawFog(!landScape.drawFog());
        		}
			});
		optionsPanel.add(cb);
		buttonPanel.add(optionsPanel);
		
		//	Create increase and decrease detail buttons.
		JPanel detailPanel = new JPanel(new GridLayout(0,1));
		detailPanel.setBorder(BorderFactory.createTitledBorder(
								BorderFactory.createLineBorder(Color.black),
								"Detail"));
		JButton button = new JButton("Increase");
		button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int detail = landScape.getLevelOfDetail() - 1;
					if (detail < 1)		detail = 1;
            		landScape.setLevelOfDetail(detail);
            		lodField.setText(""+detail);
        		}
			});
		detailPanel.add(button);
		button = new JButton("Decrease");
		button.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int detail = landScape.getLevelOfDetail() + 1;
					if (detail > 20)	detail = 20;
            		landScape.setLevelOfDetail(detail);
            		lodField.setText("" + detail);
        		}
			});
		detailPanel.add(button);
		JPanel panel = new JPanel();
		panel.add(new JLabel("Level of Detail ="));
		lodField = new JTextField("  " + landScape.getLevelOfDetail());
		lodField.setEditable(false);
		panel.add(lodField);
		detailPanel.add(panel);
		panel = new JPanel();
		panel.add(new JLabel("# Triangles ="));
		trisRendered = new JTextField("0000000");
		trisRendered.setEditable(false);
		Timer triUpdater = new Timer(1000, new ActionListener() {
				public void actionPerformed(ActionEvent e) {
            		trisRendered.setText(""+landScape.numTrianglesRendered());
        		}
			});
		triUpdater.start();
		panel.add(trisRendered);
		detailPanel.add(panel);
		buttonPanel.add(detailPanel);
		
		
		return buttonPanel;
	} 

}


