/*
*   HeightMap -- An interface in common to all height map implementations.
*
*   Copyright (C) 2001-2004 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Lesser General Public License for more details.
*
*   You should have received a copy of the GNU Lesser General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*   Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.gl;


/**
*  This is the interface in common to all height map implementations.  A height map
*  is a rectangular grid of altitude (height) values where the spacing between grid
*  points is constant and can be represented by integer indices.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author  Joseph A. Huwaldt   Date:  April 26, 2001
*  @version July 24, 2004
**/
public interface HeightMap {

	/**
	*  Returns the altitude at the given horizontal and vertical position
	*  in the height map array.
	*
	*  @param  h   The horizontal position the height array.
	*  @param  v   The vertical position in the height array.
	*  @return The altitude value at the specific location in the height map.
	**/
	public float get(int h, int v);
	
	/**
	*  Returns the height of this altitude map.
	**/
	public int getHeight();
	
	/**
	*  Returns the width of this altitude map.
	**/
	public int getWidth();
	
	/**
	*  Returns the minimum altitude in the height map.
	**/
	public float minAlt();
	
	/**
	*  Returns the maximum altitude in the height map.
	**/
	public float maxAlt();
	
}
